@@ -6,13 +6,18 @@ from typing import TypeVar, Generic
 from enum import Enum
 from sqlalchemy.exc import SQLAlchemyError
 from sqlalchemy.sql.elements import BinaryExpression
-from sqlalchemy import and_, or_
+from sqlalchemy.orm import Query
+from sqlalchemy import and_, column, or_
 from sqlalchemy import asc
 from LibHanger.Models.fields import fields
 from LibHanger.Library.DataAccess.Base.uwDataAccess import uwDataAccess
 from LibHanger.Library.uwDeclare import uwDeclare as en
+from LibHanger.Models.saJoin import saJoin
 from LibHanger.Models.saWhere import saWhere
 from LibHanger.Models.saOrderBy import saOrderBy
+from TestClass.log_race_id import log_race_id
+from TestClass.trn_race_id import trn_race_id
+from TestClass.trn_race_result import trn_race_result
 
 T = TypeVar('T')
 
@@ -134,7 +139,6 @@ class recset(Generic[T]):
         DBセッション
         """
         
-        #return self.__session
         return self.__da.session
     
     @property
@@ -211,7 +215,7 @@ class recset(Generic[T]):
         
         return self.__fields
     
-    def __initdfrows(self):
+    def __initdfrows(self, columns:list = None):
         
         """
         dfrows初期化(Dataframe)
@@ -219,8 +223,8 @@ class recset(Generic[T]):
         
         # 列リスト生成
         colList = []
-        for col in self.__columns:
-            colList.append(col[0])
+        for col in self.__columns if columns == None else columns:
+            colList.append(col[0]) if columns == None else colList.append(col.key)
         colList.append(self.__ROWSTATE_COLNAME__)
         colList.sort()
         
@@ -228,7 +232,7 @@ class recset(Generic[T]):
         df = pd.DataFrame(columns=colList)
         
         # 主キーセット
-        if len(self.__primaryKeys) > 0:
+        if len(self.__primaryKeys) > 0 and columns == None:
             df = df.set_index(self.__primaryKeys, drop=False)
 
         # 戻り値を返す
@@ -376,14 +380,14 @@ class recset(Generic[T]):
         # 行状態をmodifiedに変更
         self.State = self.rowState.modified
         
-    def __initRows(self):
+    def __initRows(self, columns:list = None):
         
         """
         行情報をリセットする
         """
         
         self.__rows = []
-        self.__dfrows = self.__initdfrows()
+        self.__dfrows = self.__initdfrows(columns)
     
     def __deepCopyRow(self, rows:list) -> list:
         
@@ -400,7 +404,8 @@ class recset(Generic[T]):
         targetRows = copy.deepcopy(rows)
         # DataFrame化で不要な列を削除
         for rowInstance in targetRows:
-            delattr(rowInstance, self.__SA_INSTANCE_STATE__)
+            if hasattr(rowInstance, self.__SA_INSTANCE_STATE__):
+                delattr(rowInstance, self.__SA_INSTANCE_STATE__)
             
         return targetRows
 
@@ -589,6 +594,150 @@ class recset(Generic[T]):
         # 戻り値を返す
         return df
     
+    def outerjoinFilter(self, j:saJoin, w:saWhere, s:saOrderBy):
+        
+        """
+        外部結合
+        
+        Parameters
+        ----------
+        j : saJoin
+            結合条件
+        w : saWhere
+            Where条件
+        s : saOrderBy
+            ソート条件
+        """
+        
+        self.joinFilter(j, True, w, s)
+    
+    def innerjoinFilter(self, j:saJoin, w:saWhere, s:saOrderBy):
+
+        """
+        内部結合
+ 
+        Parameters
+        ----------
+        j : saJoin
+            結合条件
+        w : saWhere
+            Where条件
+        s : saOrderBy
+            ソート条件
+        """
+
+        self.joinFilter(j, False, w, s)
+
+    def joinfilterExpr(self, joinTable:dict, columns:tuple, isouter, w:saWhere, s:saOrderBy):
+        
+        """
+        テーブル結合
+
+        Parameters
+        ----------
+        joinTable : dict
+            結合対象テーブル情報
+        columns : tuple[column]
+            取得列
+        isouter : bool
+            外部結合フラグ(True:外部結合 False:内部結合)
+        w : any | asWhere
+            where条件
+        s : Any | asOrderBy
+            ソート順
+        """
+        
+        # メインデータソースと取得列
+        q = self.__da.session.query(self.modelType, *columns)
+        
+        # テーブル結合
+        for jt in joinTable:
+            jk = and_(*tuple(joinTable[jt]))
+            q = q.join(jt, jk, isouter = isouter)
+            
+        # where条件
+        # -AND
+        if w != None and len(w.andList) > 0:
+            q = q.filter(and_(*tuple(w.andList)))
+        
+        # -OR
+        if w != None and len(w.orList) > 0:
+            q = q.filter(or_(*tuple(w.orList)))
+
+        # ソート条件
+        if s != None and len(s.orderBy) > 0:
+            # keyでソート
+            sort_orderBy = sorted(s.orderBy.items())
+            for sortKey in sort_orderBy:
+                q = q.order_by(sortKey[1])
+        return q
+    
+    def joinFilter(self, j:saJoin, isouter, w:saWhere, s:saOrderBy):
+
+        """
+        結合したレコードセットをフィルタする
+        結果セットはdbから取得
+
+        Parameters
+        ----------
+        j : saJoin
+            結合条件
+        isouter : bool
+            外部結合フラグ(True:外部結合 False:内部結合)
+        w : any | asWhere
+            where条件
+        s : Any | asOrderBy
+            ソート順
+        """
+    
+        # 行情報初期化
+        self.__initRows(j.columns)
+        
+        # フィールドクラス
+        self.__fields = fields(self.__rows, self.__dfrows)
+
+        # セッションチェック
+        self.__checkSessionOpen(False, False)
+
+        # クエリ実行
+        rowIndex = 0
+        q = self.joinfilterExpr(j.joinTable, j.columns, isouter, w, s).all()
+        for row in q:
+            
+            # fieldリスト作成
+            fldList = []
+            for fldIdx in range(1, len(row._fields)):
+                fldList.append(row._fields[fldIdx])
+            fldList.append(self.__SA_INSTANCE_STATE__)
+
+            # メインテーブルの行情報コピー
+            mainRow = copy.deepcopy(row[0])
+            # 行情報のメンバ取得
+            attributes = inspect.getmembers(row[0], lambda x: not(inspect.isroutine(x)))
+            rowInfoMember = list(filter(lambda x: not(x[0].startswith("__") or x[0].startswith("_") or x[0] == "metadata" or x[0] == "registry"), attributes))
+            # 抽出列以外はメンバを削除
+            for attr in rowInfoMember:
+                if attr[0] in fldList: continue
+                delattr(mainRow, attr[0])
+
+            # 行状態をnoChangedに変更
+            mainRow.__rowState = self.rowState.noChanged
+            # joinテーブルの行情報をmainRowにマージ
+            for fldIdx in range(1, len(row._fields)):
+                setattr(mainRow, row._fields[fldIdx], row._data[fldIdx])
+            # rowsにクエリ結果を追加
+            self.__rows.append(mainRow)
+            # DataFrame - add
+            self.__addRecrowDataFrame(mainRow, rowIndex)
+            # rowIndex++
+            rowIndex += 1
+        
+        # セッションチェック
+        self.__checkSessionClose()
+        
+        # カレント行インデックス初期化
+        self.__currentRowIndex = -1
+        
     def filterExpr(self, w, s = None):
         
         """
@@ -640,6 +789,47 @@ class recset(Generic[T]):
         # 戻り値を返す
         return q
     
+    def filter(self, w, s = None):
+        
+        """
+        レコードセットをフィルタする
+        結果セットはdbから取得
+
+        Parameters
+        ----------
+        w : any | asWhere
+            where条件
+        s : Any | asOrderBy
+            ソート順
+        """
+        
+        # 行情報初期化
+        self.__initRows()
+        
+        # フィールドクラス
+        self.__fields = fields(self.__rows, self.__dfrows)
+
+        # セッションチェック
+        self.__checkSessionOpen(False, False)
+
+        # クエリ実行
+        rowIndex = 0
+        q = self.filterExpr(w, s).all()
+        for row in q:
+            # 行状態をnoChangedに変更
+            row.__rowState = self.rowState.noChanged
+            # rowsにクエリ結果を追加
+            self.__rows.append(row)
+            # DataFrame - add
+            self.__addRecrowDataFrame(row, rowIndex)
+            rowIndex += 1
+        
+        # セッションチェック
+        self.__checkSessionClose()
+        
+        # カレント行インデックス初期化
+        self.__currentRowIndex = -1
+
     def convertDataFrameExpr(self, df, w):
         
         """
@@ -688,47 +878,6 @@ class recset(Generic[T]):
 
             # 戻り値を返す
             return retExpr
-        
-    def filter(self, w, s = None):
-        
-        """
-        レコードセットをフィルタする
-        結果セットはdbから取得
-
-        Parameters
-        ----------
-        w : any | asWhere
-            where条件
-        s : Any | asOrderBy
-            ソート順
-        """
-        
-        # 行情報初期化
-        self.__initRows()
-        
-        # フィールドクラス
-        self.__fields = fields(self.__rows, self.__dfrows)
-
-        # セッションチェック
-        self.__checkSessionOpen(False, False)
-
-        # クエリ実行
-        rowIndex = 0
-        q = self.filterExpr(w, s).all()
-        for row in q:
-            # 行状態をnoChangedに変更
-            row.__rowState = self.rowState.noChanged
-            # rowsにクエリ結果を追加
-            self.__rows.append(row)
-            # DataFrame - add
-            self.__addRecrowDataFrame(row, rowIndex)
-            rowIndex += 1
-        
-        # セッションチェック
-        self.__checkSessionClose()
-        
-        # カレント行インデックス初期化
-        self.__currentRowIndex = -1
 
     def find(self, w, searchOption = findOption.cacheOnly):
         
